/**
 * Test case for define.
 * Runs with mocha.
 */
'use strict'

const define = require('../lib/define.js')
const assert = require('assert')
const { moduleProxy } = require('apemanmock')
const co = require('co')
const apemandb = require('apemandb')
const apemanmodel = require('apemanmodel')
const { SAMPLE_DEFS_DIR } = apemanmodel.paths

describe('define', function () {
  this.timeout(3000)
  let tmpDir = `${__dirname}/../tmp`
  let dbFile = `${tmpDir}/testing-db.db`
  let db

  before(() => co(function * () {
    db = apemandb({
      env: {
        DATABASE: 'testing-database01',
        USERNAME: 'foo',
        PASSWORD: 'bar',
        DIALECT: 'sqlite',
        STORAGE: dbFile
      },
      models: `${SAMPLE_DEFS_DIR}/*.json`
    })
  }))

  after(() => co(function * () {

  }))

  it('Create an instance', () => co(function * () {
    let module = define(db, {
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    assert.ok(api)
  }))

  it('Ping pong', () => co(function * () {
    let module = define(db, {
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    let pong = yield api.ping('The pong!')
    assert.equal(pong, 'The pong!')
  }))

  it('Call database function', () => co(function * () {
    let module = define(db, {
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    assert.ok(api)
    yield api.setup()
    yield api.sync()
    yield api.reset()
    let description = yield api.describe()
    assert.ok(description)
    yield api.drop()
    yield api.close()
  }))

  it('Compare spec and implement', () => co(function * () {
    let module = define(db, {
      logging: false
    })
    let implNames = Object.keys(module).filter((name) => !/^\$/.test(name))
    let specNames = Object.keys(module.$spec)
    for (let implName of implNames) {
      assert.ok(!!~specNames.indexOf(implName), `${implName} is implemented, but not described in "$spec"`)
    }
    for (let specName of specNames) {
      assert.ok(!!~implNames.indexOf(specName), `${specName} is described in "$spec", but not implemented`)
    }
  }))
})

/* global describe, before, after, it */
